On Sun, 4 Oct 2026 11:48:03 GMT, John Hendrikx <[email protected]> wrote:
>> This provides and uses a new implementation of `ExpressionHelper`, called >> `ListenerManager` with improved semantics. >> >> See also #837 for a previous attempt which instead of triggering nested >> emissions immediately (like this PR and `ExpressionHelper`) would wait until >> the current emission finishes and then start a new (non-nested) emission. >> >> # Behavior >> >> |Listener...|ExpressionHelper|ListenerManager| >> |---|---|---| >> |Invocation Order|In order they were registered, invalidation listeners >> always before change listeners|(unchanged)| >> |Removal during Notification|All listeners present when notification started >> are notified, but excluded for any nested changes|Listeners are removed >> immediately regardless of nesting| >> |Addition during Notification|Only listeners present when notification >> started are notified, but included for any nested changes|New listeners are >> never called during the current notification regardless of nesting| >> >> ## Nested notifications: >> >> | |ExpressionHelper|ListenerManager| >> |---|---|---| >> |Type|Depth first (call stack increases for each nested level)|(same)| >> |# of Calls|Listeners * Depth (using incorrect old values)|Collapses nested >> changes, skipping non-changes| >> |Vetoing Possible?|No|Yes| >> |Old Value correctness|Only for listeners called before listeners making >> nested changes|Always| >> >> # Performance >> >> |Listener|ExpressionHelper|ListenerManager| >> |---|---|---| >> |Addition|Array based, append in empty slot, resize as needed|(same)| >> |Removal|Array based, shift array, resize as needed|(same)| >> |Addition during notification|Array is copied, removing collected >> WeakListeners in the process|Appended when notification finishes| >> |Removal during notification|As above|Entry is `null`ed (to avoid moving >> elements in array that is being iterated)| >> |Notification completion with changes|-|Null entries (and collected >> WeakListeners) are removed| >> |Notifying Invalidation Listeners|1 ns each|(same)| >> |Notifying Change Listeners|1 ns each (*)|2-3 ns each| >> >> (*) a simple for loop is close to optimal, but unfortunately does not >> provide correct old values >> >> # Memory Use >> >> Does not include alignment, and assumes a 32-bit VM or one that is using >> compressed oops. >> >> |Listener|ExpressionHelper|ListenerManager|OldValueCaching ListenerManager| >> |---|---|---|---| >> |No Listeners|none|none|none| >> |Single InvalidationListener|16 bytes overhead|none|none| >> |Single ChangeListener|20 bytes overhead|none|16 bytes overhe... > > John Hendrikx has updated the pull request incrementally with one additional > commit since the last revision: > > Move note about supporting lazy evaluation to implNote modules/javafx.base/src/main/java/javafx/beans/Observable.java line 47: > 45: * <p> > 46: * These implementations mark themselves as invalid when the first > invalidation > 47: * event occurs. They do not generate anymore invalidation events until > their Suggestion: * event occurs. They do not generate any more invalidation events until their modules/javafx.base/src/main/java/javafx/beans/value/ObservableValue.java line 69: > 67: * this library mark themselves as invalid when the first invalidation > event > 68: * occurs. They do not generate any more invalidation events until their > value is > 69: * recomputed and valid again. Another hidden implementation note in the same vein as the `Observable` ones. I missed it in the previous review. Since this is a subclass that is covered by the `Observable` implementation notes, it might be correct to just remove it. ------------- PR Review Comment: https://git.openjdk.org/jfx/pull/1081#discussion_r4177706384 PR Review Comment: https://git.openjdk.org/jfx/pull/1081#discussion_r4177803201
